/****************************************************************************
Date: 11/23/97
File: kenwood.hpp
Description:
	KENWOOD radio driver

History:
	11/23/97 Initial Version
*****************************************************************************/
#ifndef KENWOOD_HPP
#define KENWOOD_HPP

#include "loctypes.h"
#include "radio.hpp"	// RADIO base class
#include "slink.hpp"	// SERIALLINK class (includes UARTPARITY definition)

//***** Frequency Steps ****************
const LONG KENWOOD_STEP_DEFAULT	= 10;	// normal freq step Hz (not xcvr)
const LONG KENWOOD_STEP_AMFM	= 100;	// freq step Hz when AM or FM
const LONG KENWOOD_STEP_XCVR	= 20;	// freq step Hz when used as XCVR
const LONG KENWOOD_BUTTONLIMIT 	= 10;	// max number of steps using mic buttons
const LONG KENWOOD_1GIGAHERTZ 	= 1000000000L; // 1GHz

//***** Message Definitions ************
const INT  KENWOOD_MAXMESSAGE	    = 40;  // Max number of chars in a message
const INT  KENWOOD_OPCODESIZE	    = 2;   // all opcodes are 2 chars
const CHAR KENWOOD_ENDOFMESSAGE	    = ';'; // End of message delimiter

//****** OpCodes ***********************
// These are used to build up commands
const CHAR* const KENWOOD_MODE		= "MD";	// set mode (+ 1 digit)
const CHAR* const KENWOOD_VFOA		= "FA"; // Set or read VFO A
const CHAR* const KENWOOD_VFOB		= "FB"; // Set or read VFO B

//***** Command Messages ****************
// These are complete commands sent to the radio
const CHAR* const KENWOOD_INFO		= "IF";	// read info
const CHAR* const KENWOOD_FINEOFF	= "FS0";// set fine step off
const CHAR* const KENWOOD_FINEON	= "FS1";// set fine step on
const CHAR* const KENWOOD_MICBUTTONUP	= "UP";	// step up
const CHAR* const KENWOOD_MICBUTTONDOWN = "DN";	// step down
const CHAR* const KENWOOD_SETRXVFOA	= "FR0";// use vfo A for rx 
const CHAR* const KENWOOD_SETRXVFOB	= "FR1";// use vfo B for rx 
const CHAR* const KENWOOD_SETTXVFOA	= "FT0";// tx use vfo A
const CHAR* const KENWOOD_SETTXVFOB	= "FT1";// tx use vfo B
const CHAR* const KENWOOD_TESTMESSAGE	= " ";  // used to see if radio is ok

//***** Response Messages *****************
// These are used to decode response messages
const INT  KENWOOD_MSGFA_CODE	= ((INT) 'F' + 'A');	// VFO A Frequency
const INT  KENWOOD_MSGIF_CODE	= ((INT) 'I' + 'F');	// IF message
const INT  KENWOOD_MSGIF_MODEOFFSET = 29;  // IF message, Mode offset
const INT  KENWOOD_MSGIF_TXRXOFFSET = 28;  // IF message, TX/RX offset
const CHAR KENWOOD_MSGIF_RX	    = '0'; // IF message TX/RX param = RX

//***** COM PORT PARAMETERS ******************************************
const USHORT	KENWOOD_TIMEOUT  = 2;	// 2 second response timer
const UINT	KENWOOD_BITRATE	 = 4800;
const UINT	KENWOOD_DATABITS = 8;
const UINT	KENWOOD_STOPBITS = 2;
const UARTPARITY KENWOOD_PARITY	 = UARTPARITY_NONE;


class KENWOOD : public RADIO {
private:
	BOOL		eXCVR;		// TRUE if using both TX and RX
	BOOL		eReady;		// ready for next command

	// Rx Params
	LONG		lRxStepSize;	// Frequency Step Size
	LONG		lRxSteps;	// Rx frequency steps
	MODULATION	eRxMode;
	BOOL		eUserHasTunedRx;

	// Tx Params
	LONG		lTxStepSize;	// Frequency Step Size
	LONG		lTxSteps;	// Tx frequency steps
	MODULATION	eTxMode;	// current Tx emission type
	
	// Mode tables
	// Converts between MODULATION type and kenwood message params
	static const MODULATION aGetModeTable[]; // Kenwood to MODULATION
	static const CHAR aSetModeTable[];	 // MODULATION to Kenwood

	// ptrs to Tx and Rx set vfo commands
	const CHAR*	pszSetTxFreq;
	const CHAR*	pszSetRxFreq;

	// Serial I/O
	SERIALLINK	SerialLink;
	CHAR		aInputBuffer[KENWOOD_MAXMESSAGE]; // from radio
	CHAR		aOutputBuffer[KENWOOD_MAXMESSAGE]; // to radio

	//******************************************************
	// Utilities
	//******************************************************

	// Frequency to Frequency Steps
	LONG	FreqtoSteps(LONG lFrequency, LONG lStepSize) {
		return ( lFrequency + (lStepSize/2) ) / lStepSize;
	}

	// Set radio frequency
	VOID SetFrequency(
			DOUBLE	dNewFrequency,
			LONG 	lStepSize,
			LONG*	plSteps,
			const CHAR* pszSetVFOCmd);

	// Set radio mode
	VOID	SetMode(MODULATION eMode);

	// Get Step Size for this mode
	LONG	GetStepSize(MODULATION eMode);

private:
	// No copy constructor or assignment allowed
	// Force compile error if attempted
	KENWOOD(const KENWOOD&);
	KENWOOD& operator=(const KENWOOD&);

public:
	// General Functions
	KENWOOD(const RADIOPARAMS& Params);
	BOOL	IsReady() { return eReady; }
	VOID	Update();
	~KENWOOD();

	// Tx Functions
	VOID	SetTxFreq(DOUBLE dFrequency);
	VOID	SetTxMode(MODULATION eMode);

	// Rx Functions
	BOOL	HasUserTunedRx() { return eUserHasTunedRx; }
	VOID	RefreshRxParams();
	DOUBLE	GetRxFreq();
	MODULATION GetRxMode() { return eRxMode; }
	VOID	SetRxFreq(DOUBLE dFrequency);
	VOID	SetRxMode(MODULATION eMode);

};
#endif // end kenwood.hpp
