/****************************************************************************
Date: 12/24/98
File: radio.hpp

Description:	RADIO and RADIOSYSTEM classes
	
 	The RADIO class is the base class for the radio drivers.
	All radio drivers are derived from this class. It provides basic
	functions common to all drivers and the interface definition.

	This class can be instantiated to provide a "dummy" radio driver
	that meets the interface but does not control a real radio.
 
	To create radio objects, use the NewInstance() function. This will
	create the proper type of driver based on the RADIOPARAMS parameters.
	If the driver fails to initialize properly (i.e. radio not working,)
	the NewInstance() function returns a NULL pointer. To destroy radio
	objects, use delete().


	An object of type RADIOSYSTEM includes a RADIO object and
	transmit and receive frequency converters. The frequency
	converters can be set for no translation.

	The RADIOSYSTEM class has the same interface as the RADIO 
	classes except it has no public constructor. To create a radio system,
	use the NewInstance() function. If the RADIO driver fails to
	initialize properly (i.e. radio not working,) the NewInstance()
	function returns a NULL pointer. To destroy radio
	objects, use delete(). 
	

Public:

	//***** RADIO driver base class *********************************
	static RADIO* NewInstance(const RADIOPARAMS& Params)
	RADIO()
	virtual	~RADIO()
	BOOL		IsOperational()
	virtual BOOL	IsReady()
	virtual VOID	Update()
	virtual VOID	Debug()

	// Tx Functions
	virtual VOID	SetTxFreq(DOUBLE dFrequency)
	virtual VOID	SetTxMode(MODULATION eMode)

	// Rx Functions
	virtual BOOL	HasUserTunedRx()
	virtual VOID	RefreshRxParams() 
	virtual DOUBLE	GetRxFreq()
	virtual MODULATION GetRxMode()
	virtual VOID	SetRxFreq(DOUBLE dFrequency)
	virtual VOID	SetRxMode(MODULATION eMode)


	//***** RADIOSYSTEM class *************************************
	static RADIOSYSTEM* NewInstance(const RADIOPARAMS& Params)
	~RADIOSYSTEM()
	BOOL	IsOperational()
	BOOL	IsReady()
	VOID	Update()
	VOID	Debug()
	
	// Tx Functions
	VOID	SetTxFreq(DOUBLE dFrequency)
	VOID	SetTxMode(MODULATION eMode)
	
	//Rx Functions
	BOOL	HasUserTunedRx()
	VOID	RefreshRxParams() 
	DOUBLE	GetRxFreq()
	MODULATION GetRxMode()
	VOID	SetRxFreq(DOUBLE dFrequency)
	VOID	SetRxMode(MODULATION eMode)

History:
	12/1/97 Initial Version

*****************************************************************************/
#ifndef RADIO_HPP
#define RADIO_HPP

#include "loctypes.h"
#include "modulate.hpp"	// MODULATION type

// RADIO TYPES
enum RADIOTYPE {
	RADIOTYPE_NULL = 0,	// "dummy" radio - non functional
	RADIOTYPE_KENWOOD,	// Kenwood TS450, TS690, TS850, TS950
	RADIOTYPE_MICBUTTON,	// UP/DOWN Mic Button control Radio
	RADIOTYPE_FT736,	// Yaesu FT-736
	RADIOTYPE_FT847,	// Yaesu FT-847
	

	// Not yet available
	RADIOTYPE_TS790,	// Kenwood TS790
	RADIOTYPE_ICOM,		// Icom CI-IV or CI-V Radios
	RADIOTYPE_IC735,	// Icom IC-735
	RADIOTYPE_ICR7000,	// Icom IC-R7000
	RADIOTYPE_IC970,	// Icom  IC-970 and IC-820/821
};

// Radio Parameters
struct RADIOPARAMS {

	RADIOTYPE	eType;		// Manufacturer/Model
	BOOL		eTransceive;	// TRUE if using both TX and RX

	// I/O Params
	USHORT		unPort; 	// COM or LPT Port number
	IOADDRESS	Address;	// IO port Base Address

	// Serial port Radios only
	USHORT		unIRQ;		// IRQ Number
	USHORT		unRate;		// Port bit Rate
	USHORT		unDestination;	// Radio Destination Address (ICOM)

	// Parallel port radios only
	ULONG		ulInitialFreq;	// initial frequency
	ULONG		ulFreqStep;	// frequency step size

	// Frequency Converters
	DOUBLE		dRxConverter;
	DOUBLE		dTxConverter;
};


//***** Radio driver base class ****************************
class RADIO {
private:
	// no copy or assignment allowed
	RADIO(const RADIO&);
	RADIO& operator=(const RADIO&);

protected:
	// Basic attributes
	BOOL		eOperational;   // FALSE if could not init driver

public:
	static RADIO* NewInstance(const RADIOPARAMS& Params);
 	RADIO() : eOperational(BOOL_TRUE) { }
	virtual ~RADIO() {}
	BOOL	IsOperational() { return eOperational; } // false if init fails
	virtual BOOL	IsReady() { return BOOL_TRUE; }
	virtual VOID	Update() {}
	virtual VOID	Debug() {}

	// Tx Functions
	virtual VOID	SetTxFreq(DOUBLE dFrequency) {}
	virtual VOID	SetTxMode(MODULATION eMode) {}

	// Rx Functions
	virtual BOOL	HasUserTunedRx() { return BOOL_FALSE; }
	virtual VOID	RefreshRxParams() {}
	virtual DOUBLE	GetRxFreq() { return 0.0; }
	virtual MODULATION GetRxMode() { return MODULATION_NA; }
	virtual VOID	SetRxFreq(DOUBLE dFrequency) {}
	virtual VOID	SetRxMode(MODULATION eMode) {}

}; // end RADIO driver base class

class RADIOSYSTEM {
private:
	RADIO* pRadio;			// pointer to radio driver
	DOUBLE	   dRxConverter;	// Rx frequency translation
	DOUBLE	   dTxConverter;	// Tx frequency translation

	// no copy or assignment allowed
	RADIOSYSTEM(const RADIOSYSTEM&);
	RADIOSYSTEM& operator=(const RADIOSYSTEM&);

	// constructor
	RADIOSYSTEM(RADIO* pRadio, const RADIOPARAMS& Params) :
		pRadio(pRadio),
		dRxConverter(Params.dRxConverter),
		dTxConverter(Params.dTxConverter) { }

public:
	static RADIOSYSTEM* NewInstance(const RADIOPARAMS& Params);
	~RADIOSYSTEM() { delete pRadio; }
	BOOL	IsOperational() { return pRadio->IsOperational(); }
	BOOL	IsReady() { return pRadio->IsReady(); }
	VOID	Update() { pRadio->Update(); }
	VOID	Debug() { pRadio->Debug(); }

	// Tx Functions
	VOID	SetTxFreq(DOUBLE dFrequency) { 
		pRadio->SetTxFreq(dFrequency - dTxConverter);
	}
	VOID	SetTxMode(MODULATION eMode) { pRadio->SetTxMode(eMode); }

	// Rx Functions
	BOOL	HasUserTunedRx() { return pRadio->HasUserTunedRx(); }
	VOID	RefreshRxParams() { pRadio->RefreshRxParams(); }
	DOUBLE	GetRxFreq() { return dRxConverter + pRadio->GetRxFreq(); }
	MODULATION GetRxMode() { return pRadio->GetRxMode(); }
	VOID	SetRxFreq(DOUBLE dFrequency) { 
		pRadio->SetRxFreq(dFrequency - dRxConverter);
	}
	VOID	SetRxMode(MODULATION eMode) { pRadio->SetRxMode(eMode); }

}; // end RADIOSYSTEM class
#endif // radio.hpp
