/**************************************************************************
		Copyright 1996 - A. J. Monteiro
date: 11/22/96
file: uart8250.hpp

title:          UART8250 - I/O Interface for 8250 UART devices    
  
public:
	// General
	VOID	Open(IOADDRESS Address) 	// Initialize the UART 
	VOID	Close();			// deactivate

        // Modem Status
        BOOL    IsRLSD()                // Carrier Detect
        BOOL    IsRI()                  // Ring Indicator
        BOOL    IsDSR()                 // Data Set Ready
        BOOL    IsCTS()                 // Clear To Send

        // Modem Control
        VOID    SetModemLeads() 	// set leads for normal operation
        VOID    ClearModemLeads() 	// clear all leads

        // Line Status
        BOOL    IsRDR()              // Receiver Data Ready
        BOOL    IsTHRE()             // Transmit Holding Reg Empty
        BOOL    IsTRE()              // Transmit Reg Empty

        // Line Control
        BOOL    SetLine(UINT unBitRate,
                                UINT unDataBits,
                                UINT unStopBits,
                                UARTPARITY eParity) 

        // Interrupts
        UARTINTID GetInterruptID() 
        VOID    EnableInt(UINT unSelect)	// use UARTINTENABLE enums
        VOID    DisableInt(UINT unSelect)	// use UARTINTENABLE enums
        VOID    ClearInterrupts() 

        // Data
        VOID    PutData(char c) 
        INT     GetData() 
      
description:
Provides i/o interface to 8250 UART devices
In order to maximize run-time performance, all functions are inline
and this class is not derived from the UARTBASE base class, only
definitions from the base class are used.

If virtual UART support is needed, just define the symbol UARTBASE_VIRTUAL.

history:  AJM initial version
	 8/3/99 support added for 16550 
**************************************************************************/
#ifndef UART8250_HPP
#define UART8250_HPP

#pragma check_stack( off )
#pragma check_pointer( off )

#include "loctypes.h"	// local types
#include "uartbase.hpp" // UARTBASE abstract base class for UART Devices

#include "dospc.h"      // non-standard DOS functions

class UART8250 

#ifdef UARTBASE_VIRTUAL		// if defined, derive from base class
	: public UARTBASE 
#endif

{
private:

        //***** 8250 UART Registers ************************
        enum REG {
                 REG_DATA            = 0,
                 REG_INTENABLE       = 1,
                 REG_INTID           = 2,
                 REG_LINECONTROL     = 3,
                 REG_MODEMCONTROL    = 4,
                 REG_LINESTATUS      = 5,
                 REG_MODEMSTATUS     = 6
        };

        //***** Line Control Register **********************
        enum LINECONTROL {
                LINECONTROL_SETBITRATE  = 0x80, // 8250 UART "DLAB"
                LINECONTROL_SENDBREAK   = 0x40
        };

        //***** Modem Control Register *********************
        enum MODEMCONTROL {
                MODEMCONTROL_CLEAR   = 0x00,     // all leads off
                MODEMCONTROL_LOOPBACK= 0x10,
                MODEMCONTROL_OUT2    = 0x08,    
                MODEMCONTROL_OUT1    = 0x04,
                MODEMCONTROL_RTS     = 0x02,
                MODEMCONTROL_DTR     = 0x01,
                MODEMCONTROL_NORMAL  = 0x0f     // normal leads on
        };

        //***** Modem Status Register **********************
        enum MODEMSTATUS {
                MODEMSTATUS_RLSD     = 0x80,
                MODEMSTATUS_RI       = 0x40,
                MODEMSTATUS_DSR      = 0x20,
                MODEMSTATUS_CTS      = 0x10,
                MODEMSTATUS_RLSDCHANGE=0x08,
                MODEMSTATUS_RICHANGE = 0x04,
                MODEMSTATUS_DSRCHANGE= 0x02,
                MODEMSTATUS_CTSCHANGE= 0x01
        };
        
        //***** Line Status Register ***********************
        enum LINESTATUS {
                LINESTATUS_BREAK     = 0x10,
                LINESTATUS_FE        = 0x80,   // Frame Error
                LINESTATUS_OE        = 0x20,   // Overrun Error
                LINESTATUS_PE        = 0x40,   // Parity Error 
                LINESTATUS_RDR       = 0x01,   // Rx Data Ready
                LINESTATUS_THRE      = 0x20,   // Tx Holding Reg Empty
                LINESTATUS_TRE       = 0x40    // Tx Reg Empty
        };


        //***** UART BASE ADDRESS *********************************
        IOADDRESS BaseAddress;


        //***** UART Register I/O *********************************
        INT     GetReg(REG eReg) { return inp(BaseAddress + eReg); }
        VOID    PutReg(REG eReg, UINT unData) { 
			outp(BaseAddress + eReg, unData);
	}
        BOOL    GetModemStatus(MODEMSTATUS eLead) {
		return (BOOL) (GetReg(REG_MODEMSTATUS) & eLead);
        }
        BOOL    GetLineStatus(LINESTATUS eLead) {
		return (BOOL) (GetReg(REG_LINESTATUS) & eLead);
        }

public:
        //***** General ***********************************
	VOID    Open(IOADDRESS Address) {
        	BaseAddress = Address;
        	GetReg(REG_DATA);       // read data reg to power-up port
	}

	VOID    Close() {
		ClearModemLeads();		// modem leads off
		DisableInt(UARTINTENABLE_ALL);	// All interrupts off
	}

        //***** Modem Status *******************************
	BOOL    IsRLSD() { 	// carrier detect
		return GetModemStatus(MODEMSTATUS_RLSD);
	}
	BOOL    IsRI() { // ring indicator
		return GetModemStatus(MODEMSTATUS_RI);
	}	
	BOOL    IsDSR() { return GetModemStatus(MODEMSTATUS_DSR); }
	BOOL    IsCTS() { return GetModemStatus(MODEMSTATUS_CTS); }

        //***** Modem Control ******************************
        VOID    SetModemLeads() { 
		PutReg(REG_MODEMCONTROL, MODEMCONTROL_NORMAL);
	}
        VOID    ClearModemLeads() { 
		PutReg(REG_MODEMCONTROL, MODEMCONTROL_CLEAR);
	}

        //***** Line Status ********************************
        BOOL    IsRDR() { return GetLineStatus(LINESTATUS_RDR); }
	BOOL	IsTHRE() { return GetLineStatus(LINESTATUS_THRE); }
	BOOL	IsTRE() { return GetLineStatus(LINESTATUS_TRE); }

        //***** Line Control *******************************
	VOID    SetLine(
                        UINT unBitRate,
                        UINT unDataBits,
                        UINT unStopBits,
                        UARTPARITY eParity) {

	       //***** Set Bit Rate ********************************
	       PutReg(REG_LINECONTROL, LINECONTROL_SETBITRATE);
	       PutReg(REG_DATA, 
			((UCHAR) ((115200L/((ULONG) unBitRate)) & 0xff)) );
	       PutReg(REG_INTENABLE,
			((UCHAR) ((115200L/((ULONG) unBitRate)) >> 8)) );


	       //***** Make Line control byte **********************
	       UINT unLineControl = ((unStopBits - 1) << 2) & 0x04;
	       unLineControl |= (unDataBits - 5) & 0x03; 
	       unLineControl |= eParity;

	       //***** Reset "DLAB" and write control byte
	       PutReg(REG_LINECONTROL, (UCHAR) unLineControl);

	       return;
	}

        //***** Interrupts *********************************
        UARTINTID GetInterruptID() { return (UARTINTID) 
					(GetReg(REG_INTID) & 0x07); }

        VOID    EnableInt(UINT unSelect) {
		UINT unIntEnable = GetReg(REG_INTENABLE);
                unIntEnable |= unSelect;
                PutReg(REG_INTENABLE, unIntEnable);
        }

        VOID    DisableInt(UINT unSelect) {
		UINT unIntEnable = GetReg(REG_INTENABLE);
                unIntEnable &= ~unSelect;
                PutReg(REG_INTENABLE, unIntEnable);
        }

        VOID    ClearInterrupts() {
        	GetReg(REG_MODEMSTATUS);
        	GetReg(REG_LINESTATUS);
        	GetReg(REG_DATA);
		GetReg(REG_INTID);	// changed 8/3/99
        	//GetReg(REG_INTENABLE); // changed 8/3/99
	}

        //***** Data I/O ***************************************
        VOID    PutData(char c) { PutReg(REG_DATA, (UINT) c); }
        INT     GetData() { return GetReg(REG_DATA); }

};
#endif // uart8250.hpp
